@@ -8,11 +8,11 @@ cd luv
 git submodule update --init --recursive
 git submodule update --recursive
 
-sudo add-apt-repository --yes ppa:kalakris/cmake
-sudo apt-get update -qq
-sudo apt-get install cmake
-
-make
-sudo cp luv.so /usr/local/lib/lua/`luajit -e "_,_,v=string.find(_VERSION,'Lua (.+)');print(v)"`
+if [ "$PLATFORM" == "linux" ]; then
+  sudo add-apt-repository --yes ppa:kalakris/cmake
+  sudo apt-get update -qq
+  sudo apt-get install cmake
+fi
 
+cp luv.so $TRAVIS_BUILD_DIR
 cd $TRAVIS_BUILD_DIR
