@@ -11,9 +11,6 @@ Sometime when you want to custome x509, you maybe need to use this.
 #include "openssl.h"
 #include "private.h"
 #include <openssl/asn1.h>
-#define MYNAME    "asn1"
-#define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
-  "based on OpenSSL " SHLIB_VERSION_NUMBER
 
 #ifdef WIN32
 #define timezone _timezone
@@ -142,21 +139,34 @@ static int openssl_get_object(lua_State*L)
 {
   size_t l = 0;
   const char* asn1s = luaL_checklstring(L, 1, &l);
-  size_t off = posrelat(luaL_optinteger(L, 2, 1), l);
-  size_t length = posrelat(luaL_optinteger(L, 3, l), l);
+  size_t start = posrelat(luaL_optinteger(L, 2, 1), l);
+  size_t stop = posrelat(luaL_optinteger(L, 3, l), l);
 
-  const unsigned char *p = (const unsigned char *)asn1s + off - 1;
+  const unsigned char *p = (const unsigned char *)asn1s + start - 1;
   long len = 0;
   int tag = 0;
   int class = 0;
   int ret;
 
-  p = (const unsigned char *)asn1s + off - 1;
-  ret = ASN1_get_object(&p, &len, &tag, &class, length - off + 1);
+  if (start > l)
+  {
+    lua_pushnil(L);
+    openssl_pushargerror(L, 2, "out of range");
+    return 2;
+  }
+  if (start>stop)
+  {
+    lua_pushnil(L);
+    openssl_pushargerror(L, 3, "before of start");
+    return 2;
+  }
+
+  p = (const unsigned char *)asn1s + start - 1;
+  ret = ASN1_get_object(&p, &len, &tag, &class, stop - start + 1);
   if (ret & 0x80)
   {
     lua_pushnil(L);
-    lua_pushstring(L, "arg 1 with error encoding");
+    lua_pushstring(L, "arg #1 with error encoding");
     lua_pushinteger(L, ret);
     return 3;
   }
@@ -190,12 +200,12 @@ static int openssl_put_object(lua_State*L)
   luaL_Buffer B;
 
   luaL_argcheck(L,
-                lua_isnoneornil(L, 3) || lua_type(L, 3) == LUA_TNUMBER || lua_isstring(L, 3), 3,
+                lua_isnone(L, 3) || lua_type(L, 3) == LUA_TNUMBER || lua_isstring(L, 3), 3,
                 "if exist only accept string or number");
-  luaL_argcheck(L, lua_isnoneornil(L, 4) || lua_isboolean(L, 4), 4,
+  luaL_argcheck(L, lua_isnone(L, 4) || lua_isboolean(L, 4), 4,
                 "if exist must be boolean type");
 
-  if (lua_isnoneornil(L, 3))
+  if (lua_isnone(L, 3))
   {
     /* constructed == 2 for indefinite length constructed */
     constructed = 2;
@@ -214,7 +224,7 @@ static int openssl_put_object(lua_State*L)
       length = (int)l;
     }
 
-    constructed = lua_isnoneornil(L, 4) ? 0 : lua_toboolean(L, 4);
+    constructed = lua_isnone(L, 4) ? 0 : lua_toboolean(L, 4);
   }
   luaL_buffinit(L, &B);
   p1 = (unsigned char *)luaL_prepbuffer(&B);
@@ -323,8 +333,11 @@ static int openssl_asn1int_new(lua_State* L)
   else if (!lua_isnone(L, 1))
   {
     BIGNUM *bn = BN_get(L, 1);
-    ai = BN_to_ASN1_INTEGER(bn, ai);
-    BN_free(bn);
+    if(bn!=NULL)
+    {
+      ai = BN_to_ASN1_INTEGER(bn, ai);
+      BN_free(bn);
+    }
   }
   PUSH_OBJECT(ai, "openssl.asn1_integer");
   return 1;
@@ -504,8 +517,13 @@ static int openssl_asn1object_new(lua_State* L)
     else
       luaL_argerror(L, 1, "create object fail");
   }
+  else if(lua_isnone(L, 1))
+  {
+    ASN1_OBJECT* obj = ASN1_OBJECT_new();
+    PUSH_OBJECT(obj, "openssl.asn1_object");
+  }
   else
-    luaL_argerror(L, 1, "need accept paramater");
+    luaL_argerror(L, 1, "need accept paramater or none");
 
   return 1;
 }
@@ -885,6 +903,46 @@ static int openssl_asn1object_dup(lua_State* L)
   return 1;
 }
 
+/***
+read der in to asn1_object
+
+@function d2i
+@treturn boolean
+*/
+static int openssl_asn1object_d2i(lua_State* L)
+{
+  size_t l;
+  ASN1_OBJECT* o = CHECK_OBJECT(1, ASN1_OBJECT, "openssl.asn1_object");
+  const unsigned char* p = (const unsigned char*) luaL_checklstring(L, 2, &l);
+
+  lua_pushboolean(L, d2i_ASN1_OBJECT(&o, &p, l)!=NULL);
+  return 1;
+}
+
+/***
+get der encoded of asn1_object
+
+@function i2d
+@treturn string
+*/
+static int openssl_asn1object_i2d(lua_State* L)
+{
+  ASN1_OBJECT* o = CHECK_OBJECT(1, ASN1_OBJECT, "openssl.asn1_object");
+  int ret = i2d_ASN1_OBJECT(o, NULL);
+  if (ret > 0)
+  {
+    unsigned char *p, *O;
+    p = O = OPENSSL_malloc(ret);
+    ret = i2d_ASN1_OBJECT(o, &p);
+    if (ret>0)
+      lua_pushlstring(L, (const char*)O, ret);
+    OPENSSL_free(O);
+  }
+  if (ret==0)
+    lua_pushnil(L);
+  return 1;
+}
+
 static luaL_Reg asn1obj_funcs[] =
 {
   {"nid",         openssl_asn1object_nid},
@@ -895,6 +953,8 @@ static luaL_Reg asn1obj_funcs[] =
   {"dup",         openssl_asn1object_dup},
   {"data",        openssl_asn1object_data},
   {"equals",      openssl_asn1object_equals},
+  {"d2i",         openssl_asn1object_d2i},
+  {"i2d",         openssl_asn1object_i2d},
 
   {"__eq",        openssl_asn1object_equals},
   {"__gc",        openssl_asn1object_free},
@@ -1203,7 +1263,7 @@ static int openssl_asn1group_data(lua_State* L)
 {
   ASN1_STRING* s = CHECK_GROUP(1, ASN1_STRING, "openssl.asn1group");
   if (lua_isnone(L, 2))
-    lua_pushlstring(L, (const char*)ASN1_STRING_data(s), ASN1_STRING_length(s));
+    lua_pushlstring(L, (const char*)ASN1_STRING_get0_data(s), ASN1_STRING_length(s));
   else
   {
     size_t l;
@@ -1275,7 +1335,7 @@ static int openssl_asn1group_tostring(lua_State* L)
     case V_ASN1_INTEGER:
     case V_ASN1_BIT_STRING:
     {
-      BIGNUM *bn = BN_bin2bn((const unsigned char*)ASN1_STRING_data(s), ASN1_STRING_length(s), NULL);
+      BIGNUM *bn = BN_bin2bn((const unsigned char*)ASN1_STRING_get0_data(s), ASN1_STRING_length(s), NULL);
       char* str = BN_bn2hex(bn);
       lua_pushstring(L, str);
       BN_free(bn);
@@ -1283,7 +1343,7 @@ static int openssl_asn1group_tostring(lua_State* L)
       return 1;
     }
     default:
-      lua_pushlstring(L, (const char*) ASN1_STRING_data(s), ASN1_STRING_length(s));
+      lua_pushlstring(L, (const char*)ASN1_STRING_get0_data(s), ASN1_STRING_length(s));
       return 1;
     }
   }
@@ -1428,43 +1488,37 @@ int luaopen_asn1(lua_State *L)
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
 
-  lua_pushliteral(L, "version");    /** version */
-  lua_pushliteral(L, MYVERSION);
-  lua_settable(L, -3);
-
   auxiliar_enumerate(L, -1, asn1_const);
 
   return 1;
 }
 
-int openssl_get_nid(lua_State*L, int idx)
+ASN1_OBJECT* openssl_get_asn1object(lua_State*L, int idx, int nil)
 {
+  ASN1_OBJECT *obj = NULL;
   if (lua_type(L, idx) == LUA_TNUMBER)
   {
-    return luaL_checkint(L, idx);
+    obj = OBJ_nid2obj(luaL_checkint(L, idx));
   }
   else if (lua_isstring(L, idx))
   {
-    int nid = NID_undef;
-    ASN1_OBJECT* obj = OBJ_txt2obj(lua_tostring(L, idx), 0);
-    if (obj)
-    {
-      nid = OBJ_obj2nid(obj);
-      ASN1_OBJECT_free(obj);
-    }
-    return nid;
+    obj = OBJ_txt2obj(lua_tostring(L, idx), 0);
   }
-  else if (lua_isuserdata(L, idx))
+  else if (auxiliar_getclassudata(L, "openssl.asn1_object", idx)!=NULL)
   {
-    ASN1_OBJECT* obj = CHECK_OBJECT(idx, ASN1_OBJECT, "openssl.asn1_object");
-    return  OBJ_obj2nid(obj);
+    ASN1_OBJECT *in = CHECK_OBJECT(idx, ASN1_OBJECT, "openssl.asn1_object");
+    obj = OBJ_dup(in);
   }
-  else
+
+  if(nil!=0)
+    return obj;
+
+  if(obj==NULL)
   {
-    luaL_checkany(L, idx);
-    luaL_argerror(L, idx, "not accept paramater");
+    lua_pushfstring(L, "%s is invalid asn1_object or object id", lua_tostring(L, idx));
+    luaL_argerror(L, idx, lua_tostring(L, -1));
   }
-  return NID_undef;
+  return obj;
 }
 
 int openssl_push_asn1object(lua_State* L, const ASN1_OBJECT* obj)
@@ -1505,7 +1559,7 @@ int openssl_push_asn1(lua_State* L, const ASN1_STRING* string, int type)
   case V_ASN1_GENERALIZEDTIME:
   {
     ASN1_TIME* dup = (ASN1_TIME*) ASN1_STRING_dup(string);
-    PUSH_OBJECT(dup , "openssl.asn1_time");
+    PUSH_OBJECT(dup, "openssl.asn1_time");
     return 1;
   }
   case V_ASN1_OCTET_STRING:
