@@ -8,12 +8,9 @@ openssl.bio is a help object, it is useful, but rarely use.
 */
 #include "openssl.h"
 #include "private.h"
+#include <openssl/bn.h>
 #include <openssl/ssl.h>
 
-#define MYNAME    "bio"
-#define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
-  "based on OpenSSL " SHLIB_VERSION_NUMBER
-
 /*
 static const int* iMethods[] = {
   BIO_TYPE_NONE,
@@ -197,7 +194,7 @@ static int openssl_bio_new_connect(lua_State *L)
     }
     else
     {
-      doconn = lua_isnoneornil(L, 3) ? doconn : auxiliar_checkboolean(L, 3);
+      doconn = lua_isnone(L, 3) ? doconn : auxiliar_checkboolean(L, 3);
     }
   }
   else
@@ -634,7 +631,7 @@ setup ready and accept client connect
 static LUA_FUNCTION(openssl_bio_accept)
 {
   BIO* bio = CHECK_OBJECT(1, BIO, "openssl.bio");
-  int first = lua_isnoneornil(L, 2) ? 0 : lua_toboolean(L, 2);
+  int first = lua_isnone(L, 2) ? 0 : lua_toboolean(L, 2);
   int ret = BIO_do_accept(bio);
   if (ret == 1)
   {
@@ -741,7 +738,7 @@ static LUA_FUNCTION(openssl_bio_fd)
   if (typ & BIO_TYPE_FD)
   {
     int fd = -1;
-    if (!lua_isnoneornil(L, 2))
+    if (!lua_isnone(L, 2))
     {
       fd = lua_tointeger(L, 2);
       BIO_set_fd(bio, fd, BIO_NOCLOSE);
@@ -755,6 +752,7 @@ static LUA_FUNCTION(openssl_bio_fd)
   return 1;
 }
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
 void BIO_info_callback(BIO *bio, int cmd, const char *argp,
                        int argi, long argl, long ret)
 {
@@ -776,21 +774,21 @@ void BIO_info_callback(BIO *bio, int cmd, const char *argp,
   case BIO_CB_READ:
     if (BIO_method_type(bio) & BIO_TYPE_DESCRIPTOR)
       snprintf(p, p_maxlen, "read(%lu,%lu) - %s fd=%lu\n",
-               BIO_number_read(bio), (unsigned long)argi,
-               BIO_method_name(bio), BIO_number_read(bio));
+               (unsigned long)BIO_number_read(bio), (unsigned long)argi,
+               BIO_method_name(bio), (unsigned long)BIO_number_read(bio));
     else
       snprintf(p, p_maxlen, "read(%lu,%lu) - %s\n",
-               BIO_number_read(bio), (unsigned long)argi,
+               (unsigned long)BIO_number_read(bio), (unsigned long)argi,
                BIO_method_name(bio));
     break;
   case BIO_CB_WRITE:
     if (BIO_method_type(bio) & BIO_TYPE_DESCRIPTOR)
       snprintf(p, p_maxlen, "write(%lu,%lu) - %s fd=%lu\n",
-               BIO_number_written(bio), (unsigned long)argi,
-               BIO_method_name(bio), BIO_number_written(bio));
+               (unsigned long)BIO_number_written(bio), (unsigned long)argi,
+               BIO_method_name(bio), (unsigned long)BIO_number_written(bio));
     else
       snprintf(p, p_maxlen, "write(%lu,%lu) - %s\n",
-               BIO_number_written(bio), (unsigned long)argi,
+               (unsigned long)BIO_number_written(bio), (unsigned long)argi,
                BIO_method_name(bio));
     break;
   case BIO_CB_PUTS:
@@ -847,6 +845,7 @@ static LUA_FUNCTION(openssl_bio_set_callback)
   ret = BIO_set_info_callback(bio, BIO_info_callback);
   return openssl_pushresult(L, ret);
 }
+#endif
 
 /***
 return pending length of bytes to read and write
@@ -888,7 +887,9 @@ static luaL_Reg bio_funs[] =
   {"reset", openssl_bio_reset },
   {"retry", openssl_bio_retry },
   {"pending", openssl_bio_pending },
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
   {"set_callback", openssl_bio_set_callback },
+#endif
 
   /* for filter bio */
   {"push",  openssl_bio_push  },
@@ -935,9 +936,6 @@ int luaopen_bio(lua_State *L)
 
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
-  lua_pushliteral(L, "version");    /** version */
-  lua_pushliteral(L, MYVERSION);
-  lua_settable(L, -3);
 
   return 1;
 }
