@@ -29,13 +29,9 @@ OpenSSL not give full document about CMS api, so some function will be dangers.
 */
 #include "openssl.h"
 #include "private.h"
-#if OPENSSL_VERSION_NUMBER > 0x00909000L && !defined (LIBRESSL_VERSION_NUMBER)
+#ifndef OPENSSL_NO_CMS
 #include <openssl/cms.h>
 
-#define MYNAME    "cms"
-#define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
-  "based on OpenSSL " SHLIB_VERSION_NUMBER
-
 static LuaL_Enumeration cms_flags[] =
 {
   {"text",                  0x1},
@@ -97,6 +93,8 @@ static int openssl_cms_read(lua_State *L)
     cms = SMIME_read_CMS(in, &data);
   }
 
+  BIO_free(in);
+
   if (cms)
   {
     PUSH_OBJECT(cms, "openssl.cms");
@@ -195,6 +193,7 @@ static int openssl_cms_create(lua_State*L)
       int flags = luaL_optint(L, 2, 0);
       cms = CMS_data_create(in, flags);
     }
+    BIO_free(in);
   }
 
   PUSH_OBJECT(cms, "openssl.cms");
@@ -225,6 +224,7 @@ static int openssl_cms_compress(lua_State *L)
   flags = luaL_optint(L, 3, 0);
 
   cms = CMS_compress(in, nid, flags);
+  BIO_free(in);
 
   if (cms)
   {
@@ -251,6 +251,8 @@ static int openssl_cms_uncompress(lua_State *L)
   int flags = luaL_optint(L, 4, 0);
 
   int ret = CMS_uncompress(cms, in, out, flags);
+  BIO_free(in);
+  BIO_free(out);
   return openssl_pushresult(L, ret);
 }
 
@@ -275,6 +277,8 @@ static int openssl_cms_sign(lua_State *L)
   CMS_ContentInfo *cms;
 
   cms = CMS_sign(signcert, pkey, certs, data, flags);
+  BIO_free(data);
+  sk_X509_pop_free(certs, X509_free);
   if (cms)
   {
     PUSH_OBJECT(cms, "openssl.cms");
@@ -310,6 +314,7 @@ static int openssl_cms_verify(lua_State *L)
     BIO_get_mem_ptr(out, &mem);
     lua_pushlstring(L, mem->data, mem->length);
   }
+  sk_X509_pop_free(signers, X509_free);
   if (in!=NULL)
     BIO_free(in);
   if (out!=NULL)
@@ -372,8 +377,7 @@ static int openssl_cms_EncryptedData_decrypt(lua_State*L)
     BIO_get_mem_ptr(out, &mem);
     lua_pushlstring(L, mem->data, mem->length);
   }
-  if(dcont!=NULL)
-    BIO_free(dcont);
+  BIO_free(dcont);
   BIO_free(out);
   if(ret!=1)
     ret = openssl_pushresult(L, ret);
@@ -427,8 +431,7 @@ static int openssl_cms_digest_verify(lua_State*L)
     BIO_get_mem_ptr(out, &mem);
     lua_pushlstring(L, mem->data, mem->length);
   }
-  if(dcont!=NULL)
-    BIO_free(dcont);
+  BIO_free(dcont);
   BIO_free(out);
   if(ret!=1)
     ret = openssl_pushresult(L, ret);
@@ -543,6 +546,8 @@ static int openssl_cms_encrypt(lua_State *L)
         ret = CMS_final(cms, in, NULL, flags);
     }
   }
+  BIO_free(in);
+  sk_X509_pop_free(encerts, X509_free);
   if (ret)
   {
     PUSH_OBJECT(cms, "openssl.cms");
@@ -619,10 +624,9 @@ static int openssl_cms_decrypt(lua_State *L)
     lua_pushlstring(L, mem->data, mem->length);
   }
 
-  if (dcont!=NULL)
+  if (dcont)
     BIO_free(dcont);
-  if (out!=NULL)
-    BIO_free(out);
+  BIO_free(out);
 
   if (ret!=1)
     return openssl_pushresult(L, ret);
@@ -766,7 +770,7 @@ static int openssl_cms_content(lua_State *L)
   if (content && *content)
   {
     ASN1_OCTET_STRING* s = *content;
-    lua_pushlstring(L, (const char*)ASN1_STRING_data(s), ASN1_STRING_length(s));
+    lua_pushlstring(L, (const char*)ASN1_STRING_get0_data(s), ASN1_STRING_length(s));
   }
   else
     lua_pushnil(L);
@@ -780,6 +784,7 @@ static int openssl_cms_get_signers(lua_State*L)
   if (signers)
   {
     openssl_sk_x509_totable(L, signers);
+    sk_X509_free(signers);
     return 1;
   }
   return 0;
@@ -791,6 +796,7 @@ static int openssl_cms_data(lua_State *L)
   BIO *out = load_bio_object(L, 2);
   unsigned int flags = luaL_optint(L, 3, 0);
   int ret = CMS_data(cms, out, flags);
+  BIO_free(out);
   return openssl_pushresult(L, ret);
 }
 
@@ -802,6 +808,7 @@ static int openssl_cms_final(lua_State*L)
   int flags = luaL_optint(L, 3, 0);
 
   int ret = CMS_final(cms, in, NULL, flags);
+  BIO_free(in);
   return openssl_pushresult(L, ret);
 }
 
@@ -821,9 +828,11 @@ static int openssl_cms_sign_receipt(lua_State*L)
     if (srcms)
     {
       PUSH_OBJECT(srcms, "openssl.cms");
+      sk_X509_pop_free(other, X509_free);
       return 1;
     }
   }
+  sk_X509_pop_free(other, X509_free);
   return openssl_pushresult(L, 0);
 }
 
@@ -861,16 +870,13 @@ static luaL_Reg cms_ctx_funs[] =
 
 int luaopen_cms(lua_State *L)
 {
-#if OPENSSL_VERSION_NUMBER > 0x00909000L && !defined (LIBRESSL_VERSION_NUMBER)
+#ifndef OPENSSL_NO_CMS
   ERR_load_CMS_strings();
 
   auxiliar_newclass(L, "openssl.cms",  cms_ctx_funs);
 
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
-  lua_pushliteral(L, "version");
-  lua_pushliteral(L, MYVERSION);
-  lua_settable(L, -3);
 
   auxiliar_enumerate(L, -1, cms_flags);
 #else
