@@ -9,10 +9,6 @@
 #include <openssl/dh.h>
 #include <openssl/engine.h>
 
-#define MYNAME    "dh"
-#define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
-  "based on OpenSSL " SHLIB_VERSION_NUMBER
-
 static LUA_FUNCTION(openssl_dh_free)
 {
   DH* dh = CHECK_OBJECT(1, DH, "openssl.dh");
@@ -46,6 +42,7 @@ static LUA_FUNCTION(openssl_dh_parse)
 
 static int openssl_dh_set_method(lua_State *L)
 {
+#ifndef OPENSSL_NO_ENGINE
   DH* dh = CHECK_OBJECT(1, DH, "openssl.dh");
   ENGINE *e = CHECK_OBJECT(2, ENGINE, "openssl.engine");
   const DH_METHOD *m = ENGINE_get_DH(e);
@@ -54,6 +51,7 @@ static int openssl_dh_set_method(lua_State *L)
     int r = DH_set_method(dh, m);
     return openssl_pushresult(L, r);
   }
+#endif
   return 0;
 }
 
@@ -79,8 +77,6 @@ int luaopen_dh(lua_State *L)
 
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
-  lua_pushliteral(L, "version");
-  lua_pushliteral(L, MYVERSION);
-  lua_settable(L, -3);
+
   return 1;
 }
