@@ -9,10 +9,6 @@
 #include <openssl/dsa.h>
 #include <openssl/engine.h>
 
-#define MYNAME    "dsa"
-#define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
-  "based on OpenSSL " SHLIB_VERSION_NUMBER
-
 static LUA_FUNCTION(openssl_dsa_free)
 {
   DSA* dsa = CHECK_OBJECT(1, DSA, "openssl.dsa");
@@ -45,6 +41,7 @@ static LUA_FUNCTION(openssl_dsa_parse)
 
 static int openssl_dsa_set_method(lua_State *L)
 {
+#ifndef OPENSSL_NO_ENGINE
   DSA* dsa = CHECK_OBJECT(1, DSA, "openssl.dsa");
   ENGINE *e = CHECK_OBJECT(2, ENGINE, "openssl.engine");
   const DSA_METHOD *m = ENGINE_get_DSA(e);
@@ -53,6 +50,7 @@ static int openssl_dsa_set_method(lua_State *L)
     int r = DSA_set_method(dsa, m);
     return openssl_pushresult(L, r);
   }
+#endif
   return 0;
 }
 
@@ -78,8 +76,6 @@ int luaopen_dsa(lua_State *L)
 
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
-  lua_pushliteral(L, "version");
-  lua_pushliteral(L, MYVERSION);
-  lua_settable(L, -3);
+
   return 1;
 }
