@@ -51,6 +51,7 @@ static const char* const list[] =
 
 int openssl_engine(lua_State *L)
 {
+#ifndef OPENSSL_NO_ENGINE
   const ENGINE* eng = NULL;
   if (lua_isstring(L, 1))
   {
@@ -77,12 +78,15 @@ int openssl_engine(lua_State *L)
   }
   else
     lua_pushnil(L);
+#endif
   return 1;
 }
 
+#ifndef OPENSSL_NO_ENGINE
 static int openssl_engine_next(lua_State*L)
 {
   ENGINE* eng = CHECK_OBJECT(1, ENGINE, "openssl.engine");
+  ENGINE_up_ref(eng);
   eng = ENGINE_get_next(eng);
   if (eng)
   {
@@ -96,6 +100,7 @@ static int openssl_engine_next(lua_State*L)
 static int openssl_engine_prev(lua_State*L)
 {
   ENGINE* eng = CHECK_OBJECT(1, ENGINE, "openssl.engine");
+  ENGINE_up_ref(eng);
   eng = ENGINE_get_prev(eng);
   if (eng)
   {
@@ -239,7 +244,7 @@ static int openssl_engine_ctrl(lua_State*L)
   if (lua_isnumber(L, 2))
   {
     int cmd = luaL_checkint(L, 2);
-    if (lua_isnoneornil(L, 3))
+    if (lua_isnone(L, 3))
     {
       ret = ENGINE_cmd_is_executable(eng, cmd);
     }
@@ -285,7 +290,7 @@ static int openssl_engine_ctrl(lua_State*L)
 static int openssl_engine_gc(lua_State*L)
 {
   ENGINE* eng = CHECK_OBJECT(1, ENGINE, "openssl.engine");
-  (void) eng;
+  ENGINE_free(eng);
   return 0;
 }
 
@@ -504,7 +509,7 @@ static int openssl_engine_ex_data(lua_State *L)
   ENGINE* eng = CHECK_OBJECT(1, ENGINE, "openssl.engine");
   int idx;
   int ret;
-  if (lua_isnoneornil(L, 2))
+  if (lua_isnone(L, 2))
   {
     idx = ENGINE_get_ex_new_index(0, NULL, NULL, NULL, NULL);
     if (idx == -1)
@@ -516,7 +521,7 @@ static int openssl_engine_ex_data(lua_State *L)
     return 1;
   }
   idx = luaL_checkinteger(L, 2);
-  if (lua_isnoneornil(L, 3))
+  if (lua_isnone(L, 3))
   {
     void *p = ENGINE_get_ex_data(eng, idx);
     if (p)
@@ -574,9 +579,12 @@ static luaL_Reg eng_funcs[] =
 
   {NULL,      NULL},
 };
+#endif
 
 int openssl_register_engine(lua_State* L)
 {
+#ifndef OPENSSL_NO_ENGINE
   auxiliar_newclass(L, "openssl.engine", eng_funcs);
+#endif
   return 0;
 }
