@@ -287,3 +287,37 @@ int bin2hex(const unsigned char * src, char *dst, int len)
   dst[i * 2] = '\0';
   return i * 2;
 }
+
+int openssl_pusherror (lua_State *L, const char *fmt, ...)
+{
+  va_list argp;
+  va_start(argp, fmt);
+  luaL_where(L, 1);
+  lua_pushvfstring(L, fmt, argp);
+  va_end(argp);
+  lua_concat(L, 2);
+  return 1;
+}
+
+int openssl_pushargerror (lua_State *L, int arg, const char *extramsg)
+{
+  lua_Debug ar;
+  if (!lua_getstack(L, 0, &ar))  /* no stack frame? */
+    return openssl_pusherror(L, "bad argument #%d (%s)", arg, extramsg);
+  lua_getinfo(L, "n", &ar);
+  if (strcmp(ar.namewhat, "method") == 0)
+  {
+    arg--;  /* do not count 'self' */
+    if (arg == 0)  /* error is in the self argument itself? */
+      return openssl_pusherror(L, "calling '%s' on bad self (%s)",
+                               ar.name, extramsg);
+  }
+  if (ar.name == NULL)
+#if !defined(COMPAT53_C_) || LUA_VERSION_NUM == 502
+    ar.name = "?";
+#else
+    ar.name = (compat53_pushglobalfuncname(L, &ar)) ? lua_tostring(L, -1) : "?";
+#endif
+  return openssl_pusherror(L, "bad argument #%d to '%s' (%s)",
+                           arg, ar.name, extramsg);
+}
