@@ -8,10 +8,6 @@
 #include "private.h"
 #include "openssl/ocsp.h"
 
-#define MYNAME    "ocsp"
-#define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
-  "based on OpenSSL " SHLIB_VERSION_NUMBER
-
 static int openssl_ocsp_request_new(lua_State*L)
 {
   OCSP_REQUEST *req = NULL;
@@ -182,6 +178,8 @@ static int openssl_ocsp_request_sign(lua_State*L)
 
   ret = OCSP_request_sign(req, signer, pkey, md, others, sflags);
   lua_pushboolean(L, ret);
+  if (others!=NULL)
+    sk_X509_pop_free(others, X509_free);
   return 1;
 }
 
@@ -518,9 +516,5 @@ int luaopen_ocsp(lua_State *L)
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
 
-  lua_pushliteral(L, "version");    /** version */
-  lua_pushliteral(L, MYVERSION);
-  lua_settable(L, -3);
-
   return 1;
 }
