@@ -23,6 +23,9 @@
 #include <openssl/pem.h>
 #include <openssl/err.h>
 #include <openssl/conf.h>
+#if !defined(OPENSSL_NO_COMP)
+#include <openssl/comp.h>
+#endif
 #include <openssl/rand.h>
 #include <openssl/pkcs12.h>
 #include <openssl/opensslv.h>
@@ -49,11 +52,16 @@
 /*History
   2017-04-18  update to 0.7.1
   2017-08-04  update to 0.7.3
+  2019-03-24  update to 0.7.5-1
+  2019-05-19  update to 0.7.5-2
+  2019-08-20  update to 0.7.6
 */
 
 /*                              MNNFFPPS  */
-#define LOPENSSL_VERSION_NUM  0x00703000
-#define LOPENSSL_VERSION  "0.7.3"
+#define LOPENSSL_VERSION_NUM  0x0070800f
+#ifndef LOPENSSL_VERSION
+#define LOPENSSL_VERSION  "0.7.8"
+#endif
 
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
 #include <openssl/lhash.h>
