@@ -8,9 +8,10 @@ PKCS7 module for lua-openssl binding
 #include "openssl.h"
 #include "private.h"
 
-#define MYNAME    "pkcs7"
-#define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
-  "based on OpenSSL " SHLIB_VERSION_NUMBER
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) \
+ || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x2090000fL)
+#define OPENSSL_USE_M_ASN1
+#endif
 
 /***
 read string or bio object, which include pkcs7 content
@@ -346,7 +347,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     os = p7->d.signed_and_enveloped->enc_data->enc_data;
     if (!os)
     {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
       os = M_ASN1_OCTET_STRING_new();
 #else
       os = ASN1_OCTET_STRING_new();
@@ -364,7 +365,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     os = p7->d.enveloped->enc_data->enc_data;
     if (!os)
     {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
       os = M_ASN1_OCTET_STRING_new();
 #else
       os = ASN1_OCTET_STRING_new();
@@ -383,7 +384,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     /* If detached data then the content is excluded */
     if (PKCS7_type_is_data(p7->d.sign->contents) && p7->detached)
     {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
       M_ASN1_OCTET_STRING_free(os);
 #else
       ASN1_OCTET_STRING_free(os);
@@ -398,7 +399,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     /* If detached data then the content is excluded */
     if (PKCS7_type_is_data(p7->d.digest->contents) && p7->detached)
     {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
       M_ASN1_OCTET_STRING_free(os);
 #else
       ASN1_OCTET_STRING_free(os);
@@ -474,7 +475,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
       goto err;
     if (!EVP_DigestFinal_ex(mdc, md_data, &md_len))
       goto err;
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
     M_ASN1_OCTET_STRING_set(p7->d.digest->digest, md_data, md_len);
 #else
     ASN1_OCTET_STRING_set(p7->d.digest->digest, md_data, md_len);
@@ -1098,7 +1099,7 @@ static LUA_FUNCTION(openssl_pkcs7_sign_digest)
   size_t l;
   const char* data = luaL_checklstring(L, 2, &l);
   long flags = luaL_optint(L, 3, 0);
-  int hash = lua_isnoneornil(L, 4) ? 0 : lua_toboolean(L, 4);
+  int hash = lua_isnone(L, 4) ? 0 : lua_toboolean(L, 4);
 
   int ret = 0;
   int i, j;
@@ -1137,7 +1138,7 @@ static LUA_FUNCTION(openssl_pkcs7_sign_digest)
     os = p7->d.signed_and_enveloped->enc_data->enc_data;
     if (!os)
     {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
       os = M_ASN1_OCTET_STRING_new();
 #else
       os = ASN1_OCTET_STRING_new();
@@ -1155,7 +1156,7 @@ static LUA_FUNCTION(openssl_pkcs7_sign_digest)
     os = p7->d.enveloped->enc_data->enc_data;
     if (!os)
     {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
       os = M_ASN1_OCTET_STRING_new();
 #else
       os = ASN1_OCTET_STRING_new();
@@ -1174,7 +1175,7 @@ static LUA_FUNCTION(openssl_pkcs7_sign_digest)
     /* If detached data then the content is excluded */
     if (PKCS7_type_is_data(p7->d.sign->contents) && p7->detached)
     {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
       M_ASN1_OCTET_STRING_free(os);
 #else
       ASN1_OCTET_STRING_free(os);
@@ -1189,7 +1190,7 @@ static LUA_FUNCTION(openssl_pkcs7_sign_digest)
     /* If detached data then the content is excluded */
     if (PKCS7_type_is_data(p7->d.digest->contents) && p7->detached)
     {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if defined(OPENSSL_USE_M_ASN1)
       M_ASN1_OCTET_STRING_free(os);
 #else
       ASN1_OCTET_STRING_free(os);
@@ -1333,7 +1334,7 @@ static LUA_FUNCTION(openssl_pkcs7_verify_digest)
   size_t len = 0;
   const char* data = luaL_optlstring(L, 4, NULL, &len);
   long flags = luaL_optint(L, 5, 0);
-  int hash = lua_isnoneornil(L, 6) ? 0 : lua_toboolean(L, 6);
+  int hash = lua_isnone(L, 6) ? 0 : lua_toboolean(L, 6);
 
   STACK_OF(X509) *signers;
   X509 *signer;
@@ -1523,10 +1524,6 @@ int luaopen_pkcs7(lua_State *L)
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
 
-  lua_pushliteral(L, "version");    /** version */
-  lua_pushliteral(L, MYVERSION);
-  lua_settable(L, -3);
-
   auxiliar_enumerate(L, -1, pkcs7_const);
   return 1;
 }
