@@ -12,9 +12,13 @@ create and manage x509 certificate
 #define CRYPTO_LOCK_REF
 #include "sk.h"
 
-#define MYNAME    "x509"
-#define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
-  "based on OpenSSL " SHLIB_VERSION_NUMBER
+#if OPENSSL_VERSION_NUMBER < 0x1010000fL || \
+	(defined(LIBRESSL_VERSION_NUMBER) && (LIBRESSL_VERSION_NUMBER < 0x20700000L))
+#define X509_get0_notBefore X509_get_notBefore
+#define X509_get0_notAfter X509_get_notAfter
+#define X509_set1_notBefore X509_set_notBefore
+#define X509_set1_notAfter X509_set_notAfter
+#endif
 
 static int openssl_push_purpose(lua_State*L, X509_PURPOSE* purpose)
 {
@@ -43,7 +47,7 @@ get special purpose info as table
 */
 static int openssl_x509_purpose(lua_State*L)
 {
-  if (lua_isnoneornil(L, 1))
+  if (lua_isnone(L, 1))
   {
     int count = X509_PURPOSE_get_count();
     int i;
@@ -81,6 +85,9 @@ static int openssl_x509_purpose(lua_State*L)
       lua_pushnil(L);
     return 1;
   }
+  else
+    luaL_argerror(L, 1, "only accpet none, string or number as nid or short name");
+
   return 0;
 };
 
@@ -460,7 +467,7 @@ static LUA_FUNCTION(openssl_x509_export)
 {
   X509 *cert = CHECK_OBJECT(1, X509, "openssl.x509");
   int fmt = luaL_checkoption(L, 2, "pem", format);
-  int notext = lua_isnoneornil(L, 3) ? 1 : lua_toboolean(L, 3);
+  int notext = lua_isnone(L, 3) ? 1 : lua_toboolean(L, 3);
   BIO* out = NULL;
 
   if (fmt != FORMAT_DER && fmt != FORMAT_PEM)
@@ -535,9 +542,9 @@ static LUA_FUNCTION(openssl_x509_parse)
   PUSH_ASN1_INTEGER(L, X509_get0_serialNumber(cert));
   lua_setfield(L, -2, "serialNumber");
 
-  PUSH_ASN1_TIME(L, X509_get_notBefore(cert));
+  PUSH_ASN1_TIME(L, X509_get0_notBefore(cert));
   lua_setfield(L, -2, "notBefore");
-  PUSH_ASN1_TIME(L, X509_get_notAfter(cert));
+  PUSH_ASN1_TIME(L, X509_get0_notAfter(cert));
   lua_setfield(L, -2, "notAfter");
 
   {
@@ -730,6 +737,8 @@ static LUA_FUNCTION(openssl_x509_check)
 #if 0
     X509_STORE_set_verify_cb_func(store, verify_cb);
 #endif
+    if (untrustedchain!=NULL)
+      sk_X509_pop_free(untrustedchain, X509_free);
     ret = check_cert(store, cert, untrustedchain, purpose);
     lua_pushboolean(L, ret == X509_V_OK);
     lua_pushinteger(L, ret);
@@ -738,6 +747,46 @@ static LUA_FUNCTION(openssl_x509_check)
   }
 }
 
+/***
+The functions return 1 for a successful match, 0 for a failed match and -1 for
+an internal error: typically a memory allocation failure or an ASN.1 decoding
+error.
+
+All functions can also return -2 if the input is malformed. For example,
+X509_check_host() returns -2 if the provided name contains embedded NULs.
+*/
+static int openssl_push_check_result(lua_State *L, int ret, const char* name)
+{
+  switch (ret)
+  {
+  case 1:
+    lua_pushboolean(L, 1);
+    if (name)
+    {
+      lua_pushstring(L, name);
+      ret = 2;
+    }
+    break;
+  case 0:
+    lua_pushboolean(L, 0);
+    ret = 1;
+    break;
+  case -1:
+    lua_pushnil(L);
+    lua_pushliteral(L, "internal");
+    ret = 2;
+  case -2:
+    lua_pushnil(L);
+    lua_pushliteral(L, "malformed");
+    ret = 2;
+  default:
+    lua_pushnil(L);
+    lua_pushinteger(L, ret);
+    ret = 2;
+  }
+  return ret;
+}
+
 #if OPENSSL_VERSION_NUMBER > 0x10002000L
 /***
 check x509 for host (only for openssl 1.0.2 or greater)
@@ -748,17 +797,17 @@ check x509 for host (only for openssl 1.0.2 or greater)
 static LUA_FUNCTION(openssl_x509_check_host)
 {
   X509 * cert = CHECK_OBJECT(1, X509, "openssl.x509");
-  if (lua_isstring(L, 2))
-  {
-    const char *hostname = lua_tostring(L, 2);
-    lua_pushboolean(L, X509_check_host(cert, hostname, strlen(hostname), 0, NULL));
-  }
-  else
-  {
-    lua_pushboolean(L, 0);
-  }
-  return 1;
+  size_t sz;
+  const char* hostname = luaL_checklstring(L, 2, &sz);
+  int flags = luaL_optint(L, 3, 0);
+  char *peer = NULL;
+
+  int ret = X509_check_host(cert, hostname, sz, flags, &peer);
+  ret = openssl_push_check_result(L, ret, peer);
+  OPENSSL_free(peer);
+  return ret;
 }
+
 /***
 check x509 for email address (only for openssl 1.0.2 or greater)
 @tparam string email to check for match match with x509 subject
@@ -768,16 +817,11 @@ check x509 for email address (only for openssl 1.0.2 or greater)
 static LUA_FUNCTION(openssl_x509_check_email)
 {
   X509 * cert = CHECK_OBJECT(1, X509, "openssl.x509");
-  if (lua_isstring(L, 2))
-  {
-    const char *email = lua_tostring(L, 2);
-    lua_pushboolean(L, X509_check_email(cert, email, strlen(email), 0));
-  }
-  else
-  {
-    lua_pushboolean(L, 0);
-  }
-  return 1;
+  size_t sz;
+  const char *email = luaL_checklstring(L, 2, &sz);
+  int flags = luaL_optint(L, 3, 0);
+  int ret = X509_check_email(cert, email, sz, flags);
+  return openssl_push_check_result(L, ret, NULL);
 }
 
 /***
@@ -786,19 +830,14 @@ check x509 for ip address (ipv4 or ipv6, only for openssl 1.0.2 or greater)
 @tparam string ip to check for match match with x509 subject
 @treturn boolean result true if host is present and matches the certificate
 */
-static LUA_FUNCTION(openssl_x509_check_ip_asc)
+static LUA_FUNCTION(openssl_x509_check_ip)
 {
   X509 * cert = CHECK_OBJECT(1, X509, "openssl.x509");
-  if (lua_isstring(L, 2))
-  {
-    const char *ip_asc = lua_tostring(L, 2);
-    lua_pushboolean(L, X509_check_ip_asc(cert, ip_asc, 0));
-  }
-  else
-  {
-    lua_pushboolean(L, 0);
-  }
-  return 1;
+  size_t sz;
+  const char *ip = luaL_checklstring(L, 2, &sz);
+  int flags = luaL_optint(L, 3, 0);
+  int ret = X509_check_ip(cert, (const unsigned char*)ip, sz, flags);
+  return openssl_push_check_result(L, ret, NULL);
 }
 #endif
 
@@ -935,7 +974,7 @@ static int openssl_x509_notbefore(lua_State *L)
   X509* cert = CHECK_OBJECT(1, X509, "openssl.x509");
   if (lua_isnone(L, 2))
   {
-    return PUSH_ASN1_TIME(L, X509_get_notBefore(cert));
+    return PUSH_ASN1_TIME(L, X509_get0_notBefore(cert));
   }
   else
   {
@@ -959,7 +998,7 @@ static int openssl_x509_notbefore(lua_State *L)
     }
     if (at)
     {
-      ret = X509_set_notBefore(cert, at);
+      ret = X509_set1_notBefore(cert, at);
       ASN1_TIME_free(at);
     }
     else
@@ -983,7 +1022,7 @@ static int openssl_x509_notafter(lua_State *L)
   X509* cert = CHECK_OBJECT(1, X509, "openssl.x509");
   if (lua_isnone(L, 2))
   {
-    return PUSH_ASN1_TIME(L, X509_get_notAfter(cert));
+    return PUSH_ASN1_TIME(L, X509_get0_notAfter(cert));
   }
   else
   {
@@ -1007,7 +1046,7 @@ static int openssl_x509_notafter(lua_State *L)
     }
     if (at)
     {
-      ret = X509_set_notAfter(cert, at);
+      ret = X509_set1_notAfter(cert, at);
       ASN1_TIME_free(at);
     }
     else
@@ -1039,19 +1078,19 @@ static int openssl_x509_valid_at(lua_State* L)
     time_t now = 0;;
     time(&now);
 
-    lua_pushboolean(L, (X509_cmp_time(X509_get_notAfter(cert), &now)     >= 0
-                        && X509_cmp_time(X509_get_notBefore(cert), &now) <= 0));
-    PUSH_ASN1_TIME(L, X509_get_notBefore(cert));
-    PUSH_ASN1_TIME(L, X509_get_notAfter(cert));
+    lua_pushboolean(L, (X509_cmp_time(X509_get0_notAfter(cert), &now)     >= 0
+                        && X509_cmp_time(X509_get0_notBefore(cert), &now) <= 0));
+    PUSH_ASN1_TIME(L, X509_get0_notBefore(cert));
+    PUSH_ASN1_TIME(L, X509_get0_notAfter(cert));
     return 3;
   }
   else if (lua_gettop(L) == 2)
   {
     time_t time = luaL_checkinteger(L, 2);
-    lua_pushboolean(L, (X509_cmp_time(X509_get_notAfter(cert), &time)     >= 0
-                        && X509_cmp_time(X509_get_notBefore(cert), &time) <= 0));
-    PUSH_ASN1_TIME(L, X509_get_notBefore(cert));
-    PUSH_ASN1_TIME(L, X509_get_notAfter(cert));
+    lua_pushboolean(L, (X509_cmp_time(X509_get0_notAfter(cert), &time)     >= 0
+                        && X509_cmp_time(X509_get0_notBefore(cert), &time) <= 0));
+    PUSH_ASN1_TIME(L, X509_get0_notBefore(cert));
+    PUSH_ASN1_TIME(L, X509_get0_notAfter(cert));
     return 3;
   }
   else if (lua_gettop(L) == 3)
@@ -1066,9 +1105,9 @@ static int openssl_x509_valid_at(lua_State* L)
     aa = ASN1_TIME_new();
     ASN1_TIME_set(ab, before);
     ASN1_TIME_set(aa, after);
-    ret = X509_set_notBefore(cert, ab);
+    ret = X509_set1_notBefore(cert, ab);
     if (ret == 1)
-      ret = X509_set_notAfter(cert, aa);
+      ret = X509_set1_notAfter(cert, aa);
 
     ASN1_TIME_free(ab);
     ASN1_TIME_free(aa);
@@ -1214,7 +1253,7 @@ static int openssl_x509_extensions(lua_State* L)
       else
         X509_add_ext(self, ext, -1);
     }
-    sk_X509_EXTENSION_free(others);
+    sk_X509_EXTENSION_pop_free(others, X509_EXTENSION_free);
 #endif
     return openssl_pushresult(L, 1);
   }
@@ -1231,7 +1270,7 @@ sign x509
 static int openssl_x509_sign(lua_State*L)
 {
   X509* x = CHECK_OBJECT(1, X509, "openssl.x509");
-  if (lua_isnoneornil(L, 2))
+  if (lua_isnone(L, 2))
   {
     unsigned char *out = NULL;
     int len = i2d_re_X509_tbs(x, &out);
@@ -1280,7 +1319,7 @@ static int openssl_x509_sign(lua_State*L)
   {
     size_t sig_len;
     const char* sig = luaL_checklstring(L, 2, &sig_len);
-    int nid = openssl_get_nid(L, 3);
+    ASN1_OBJECT *obj = openssl_get_asn1object(L, 3, 0);
     CONSTIFY_X509_get0 ASN1_BIT_STRING *psig = NULL;
     CONSTIFY_X509_get0 X509_ALGOR *palg = NULL;
     int ret;
@@ -1289,9 +1328,10 @@ static int openssl_x509_sign(lua_State*L)
     ret = ASN1_BIT_STRING_set((ASN1_BIT_STRING*)psig, (unsigned char*)sig, (int)sig_len);
     if (ret == 1)
     {
-      ASN1_OBJECT *obj = OBJ_nid2obj(nid);
       ret = X509_ALGOR_set0((X509_ALGOR*)palg, obj, V_ASN1_UNDEF, NULL);
     }
+    else
+      ASN1_OBJECT_free(obj);
     return openssl_pushresult(L, ret);
   }
 }
@@ -1299,7 +1339,7 @@ static int openssl_x509_sign(lua_State*L)
 static int openssl_x509_verify(lua_State*L)
 {
   X509* x = CHECK_OBJECT(1, X509, "openssl.x509");
-  if (lua_isnoneornil(L, 2))
+  if (lua_isnone(L, 2))
   {
     unsigned char *out = NULL;
     int len = i2d_re_X509_tbs(x, &out);
@@ -1346,7 +1386,7 @@ static luaL_Reg x509_funcs[] =
 #if OPENSSL_VERSION_NUMBER > 0x10002000L
   {"check_host",  openssl_x509_check_host},
   {"check_email", openssl_x509_check_email},
-  {"check_ip_asc", openssl_x509_check_ip_asc},
+  {"check_ip_asc", openssl_x509_check_ip},
 #endif
   {"pubkey",      openssl_x509_public_key},
   {"version",     openssl_x509_version},
@@ -1370,6 +1410,25 @@ static luaL_Reg x509_funcs[] =
   {NULL,      NULL},
 };
 
+#if OPENSSL_VERSION_NUMBER > 0x10002000L
+static LuaL_Enumeration check_flags_const[] =
+{
+#define DEFINE_ENUM(x)  \
+  {#x,  X509_CHECK_FLAG_##x}
+  DEFINE_ENUM(ALWAYS_CHECK_SUBJECT),
+#if OPENSSL_VERSION_NUMBER > 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+  DEFINE_ENUM(NEVER_CHECK_SUBJECT),
+#endif
+  DEFINE_ENUM(NO_WILDCARDS),
+  DEFINE_ENUM(NO_PARTIAL_WILDCARDS),
+  DEFINE_ENUM(MULTI_LABEL_WILDCARDS),
+  DEFINE_ENUM(SINGLE_LABEL_SUBDOMAINS),
+#undef DEFINE_ENUM
+
+  {NULL,           0}
+};
+#endif
+
 int luaopen_x509(lua_State *L)
 {
   auxiliar_newclass(L, "openssl.x509", x509_funcs);
@@ -1393,9 +1452,12 @@ int luaopen_x509(lua_State *L)
   luaopen_x509_crl(L);
   lua_setfield(L, -2, "crl");
 
-  lua_pushliteral(L, "version");    /** version */
-  lua_pushliteral(L, MYVERSION);
+#if OPENSSL_VERSION_NUMBER > 0x10002000L
+  lua_pushliteral(L, "check_flag");
+  lua_newtable(L);
+  auxiliar_enumerate(L, -1, check_flags_const);
   lua_settable(L, -3);
+#endif
 
   return 1;
 }
