@@ -11,8 +11,6 @@ Sometime when you make CSR,TS or X509, you maybe need to use this.
 #include "sk.h"
 IMP_LUA_SK(X509_ALGOR, x509_algor)
 
-#define MYNAME "x509.algor"
-
 /***
 Create x509_algor object
 
@@ -138,11 +136,18 @@ static int openssl_xalgor_set(lua_State* L)
   int ret = 0;
   X509_ALGOR* alg = CHECK_OBJECT(1, X509_ALGOR, "openssl.x509_algor");
   ASN1_OBJECT* obj = CHECK_OBJECT(2, ASN1_OBJECT, "openssl.asn1_object");
-  ASN1_STRING* val = lua_isnoneornil(L, 3) ?
+  ASN1_STRING* val = lua_isnone(L, 3) ?
                      NULL : auxiliar_checkgroup(L, "openssl.asn1_string", 3);
   obj = OBJ_dup(obj);
-  val = ASN1_STRING_dup(val);
-  ret = X509_ALGOR_set0(alg, obj, val->type, val);
+  if(val)
+  {
+    val = ASN1_STRING_dup(val);
+    ret = X509_ALGOR_set0(alg, obj, val->type, val);
+  }
+  else
+  {
+    ret = X509_ALGOR_set0(alg, obj, 0, NULL);
+  }
   return openssl_pushresult(L, ret);
 }
 
