@@ -13,8 +13,6 @@ Sometime when you make CSR,TS or X509, you maybe need to use this.
 
 #include "sk.h"
 
-#define MYNAME "x509.extension"
-
 /***
 x509_extension contrust param table.
 
@@ -32,24 +30,22 @@ xattr = x509.attrextension.new_extension {
 */
 static X509_EXTENSION* openssl_new_xextension(lua_State*L, int idx, int v3)
 {
-  int nid;
   int critical = 0;
   ASN1_OCTET_STRING* value = NULL;
   X509_EXTENSION* y = NULL;
+  ASN1_OBJECT *obj = NULL;
+  int nid = NID_undef;
 
   lua_getfield(L, idx, "object");
-  nid = openssl_get_nid(L, -1);
+  obj = openssl_get_asn1object(L, -1, 0);
+  nid = OBJ_obj2nid(obj);
+  ASN1_OBJECT_free(obj);
   lua_pop(L, 1);
 
   lua_getfield(L, idx, "critical");
   critical = lua_isnil(L, -1) ? 0 : lua_toboolean(L, -1);
   lua_pop(L, 1);
 
-  if (nid == NID_undef)
-  {
-    lua_pushfstring(L, "%s is not valid object id", lua_tostring(L, -1));
-    luaL_argerror(L, idx, lua_tostring(L, -1));
-  }
   lua_getfield(L, idx, "value");
   if (lua_isstring(L, -1))
   {
@@ -115,6 +111,11 @@ static X509_EXTENSION* openssl_new_xextension(lua_State*L, int idx, int v3)
           else method->ext_free(ext_struc);
         }
       }
+      else
+      {
+        value = ASN1_STRING_type_new(V_ASN1_OCTET_STRING);
+        ASN1_STRING_set(value, data, size);
+      }
     }
     else
     {
@@ -258,13 +259,17 @@ static int openssl_xext_support(lua_State*L)
   else
   {
     int i;
-    int ex_nid = openssl_get_nid(L, 1);
-    if (ex_nid == NID_undef)
+    int nid = NID_undef;
+    ASN1_OBJECT *obj = openssl_get_asn1object(L, 1, 1);
+    if (obj == NULL)
       return 0;
+    else
+      nid = OBJ_obj2nid(obj);
+    ASN1_OBJECT_free(obj);
 
     for (i = 0; i < sizeof(supported_nids) / sizeof(int); i++)
     {
-      if (supported_nids[i] == ex_nid)
+      if (supported_nids[i] == nid)
         break;
     }
     lua_pushboolean(L, i < sizeof(supported_nids) / sizeof(int));
@@ -418,10 +423,9 @@ static int openssl_xext_object(lua_State* L)
   }
   else
   {
-    int nid = openssl_get_nid(L, 2);
-    int ret;
-    obj = OBJ_nid2obj(nid);
-    ret = X509_EXTENSION_set_object(x, obj);
+    ASN1_OBJECT *obj = openssl_get_asn1object(L, 2, 0);
+    int ret = X509_EXTENSION_set_object(x, obj);
+    ASN1_OBJECT_free(obj);
     return openssl_pushresult(L, ret);
   }
 };
