@@ -11,8 +11,6 @@ Sometime when you make CSR,TS or X509, you maybe need to use this.
 
 #include "sk.h"
 
-#define MYNAME "x509.name"
-
 int openssl_push_xname_asobject(lua_State*L, X509_NAME* xname)
 {
   X509_NAME* dup = X509_NAME_dup(xname);
@@ -36,18 +34,11 @@ static int openssl_new_xname(lua_State*L, X509_NAME* xname, int idx, int utf8)
     while (lua_next(L, -2) != 0)
     {
       size_t size;
-      const char *value;
-      int ret;
-      int nid = openssl_get_nid(L, -2);
-      value = luaL_checklstring(L, -1, &size);
+      ASN1_OBJECT *obj = openssl_get_asn1object(L, -2, 0);
+      const char *value = luaL_checklstring(L, -1, &size);
 
-      if (nid == NID_undef)
-      {
-        lua_pushfstring(L, "node at %d which key (%s) is not a valid object identity",
-                        i + 1, lua_tostring(L, -2));
-        luaL_argerror(L, idx, lua_tostring(L, -1));
-      }
-      ret = X509_NAME_add_entry_by_NID(xname, nid, utf8 ? MBSTRING_UTF8 : MBSTRING_ASC, (unsigned char*)value, (int)size, -1, 0);
+      int ret = X509_NAME_add_entry_by_OBJ(xname, obj, utf8 ? MBSTRING_UTF8 : MBSTRING_ASC, (unsigned char*)value, (int)size, -1, 0);
+      ASN1_OBJECT_free(obj);
       if (ret != 1)
       {
         lua_pushfstring(L, "node at %d which  %s=%s can't add to X509 name",
@@ -80,7 +71,7 @@ static int openssl_xname_new(lua_State*L)
   X509_NAME* xn;
   int utf8;
   luaL_checktable(L, 1);
-  utf8 = lua_isnoneornil(L, 2) ? 1 : lua_toboolean(L, 2);
+  utf8 = lua_isnone(L, 2) ? 1 : lua_toboolean(L, 2);
   xn = X509_NAME_new();
   openssl_new_xname(L, xn, 1, utf8);
   PUSH_OBJECT(xn, "openssl.x509_name");
@@ -224,7 +215,7 @@ static int openssl_push_xname_entry(lua_State* L, X509_NAME_ENTRY* ne, int obj)
   else
   {
     lua_pushstring(L, OBJ_nid2sn(OBJ_obj2nid(object)));
-    lua_pushlstring(L, (const char*)ASN1_STRING_data(value), ASN1_STRING_length(value));
+    lua_pushlstring(L, (const char*)ASN1_STRING_get0_data(value), ASN1_STRING_length(value));
   }
   lua_settable(L, -3);
   return 1;
@@ -241,7 +232,7 @@ return x509_name as table
 static int openssl_xname_info(lua_State*L)
 {
   X509_NAME* name = CHECK_OBJECT(1, X509_NAME, "openssl.x509_name");
-  int obj = lua_isnoneornil(L, 2) ? 0 : lua_toboolean(L, 2);
+  int obj = lua_isnone(L, 2) ? 0 : lua_toboolean(L, 2);
   int i, n;
   lua_newtable(L);
   for (i = 0, n = X509_NAME_entry_count(name); i < n; i++)
@@ -332,20 +323,19 @@ get text by given asn1_object or nid
 static int openssl_xname_get_text(lua_State*L)
 {
   X509_NAME* xn = CHECK_OBJECT(1, X509_NAME, "openssl.x509_name");
-  int nid = openssl_get_nid(L, 2);
+  ASN1_OBJECT *obj = openssl_get_asn1object(L, 2, 0);
   int lastpos = luaL_optint(L, 3, -1);
   X509_NAME_ENTRY *e;
   ASN1_STRING *s;
-  if (nid == NID_undef)
-    return 0;
 
-  lastpos = X509_NAME_get_index_by_NID(xn, nid, lastpos);
+  lastpos = X509_NAME_get_index_by_OBJ(xn, obj, lastpos);
+  ASN1_OBJECT_free(obj);
   if (lastpos == -1)
     return 0;
 
   e = X509_NAME_get_entry(xn, lastpos);
   s = X509_NAME_ENTRY_get_data(e);
-  lua_pushlstring(L, (const char *)ASN1_STRING_data(s), ASN1_STRING_length(s));
+  lua_pushlstring(L, (const char *)ASN1_STRING_get0_data(s), ASN1_STRING_length(s));
   lua_pushinteger(L, lastpos);
   return 2;
 };
@@ -361,7 +351,7 @@ static int openssl_xname_get_entry(lua_State*L)
 {
   X509_NAME* xn = CHECK_OBJECT(1, X509_NAME, "openssl.x509_name");
   int lastpos = luaL_checkint(L, 2);
-  int obj = lua_isnoneornil(L, 3) ? 0 : lua_toboolean(L, 3);
+  int obj = lua_isnone(L, 3) ? 0 : lua_toboolean(L, 3);
   X509_NAME_ENTRY *e = X509_NAME_get_entry(xn, lastpos);
   if (e)
   {
@@ -382,17 +372,13 @@ add name entry
 static int openssl_xname_add_entry(lua_State*L)
 {
   X509_NAME* xn = CHECK_OBJECT(1, X509_NAME, "openssl.x509_name");
-  int nid = openssl_get_nid(L, 2);
+  ASN1_OBJECT *obj = openssl_get_asn1object(L, 2, 0);
   size_t size;
   const char*value = luaL_checklstring(L, 3, &size);
-  int utf8 = lua_isnoneornil(L, 4) ? 1 : lua_toboolean(L, 4);
-  int ret;
-  if (nid == NID_undef)
-  {
-    lua_pushfstring(L, "(%s) is not a valid object identity",  lua_tostring(L, 2));
-    luaL_argerror(L, 2, lua_tostring(L, -1));
-  }
-  ret = X509_NAME_add_entry_by_NID(xn, nid, utf8 ? MBSTRING_UTF8 : MBSTRING_ASC, (unsigned char*)value, (int)size, -1, 0);
+  int utf8 = lua_isnone(L, 4) ? 1 : lua_toboolean(L, 4);
+
+  int ret = X509_NAME_add_entry_by_OBJ(xn, obj, utf8 ? MBSTRING_UTF8 : MBSTRING_ASC, (unsigned char*)value, (int)size, -1, 0);
+  ASN1_OBJECT_free(obj);
   if (ret != 1)
   {
     luaL_error(L,  "%s=%s can't add to X509 name",  lua_tostring(L, 2), value);
