@@ -5,7 +5,13 @@ TestEngine = {}
         local eng = assert(openssl.engine('openssl'))
         assert(eng:id(),'openssl')
         assert(eng:set_default('RSA'))
-        assert(eng:set_default('ECDSA'))
+        local _,sslv
+        _, _, sslv = openssl.version(true)
+        if sslv>=0x10100000 then
+          assert(eng:set_default('EC'))
+        else
+          assert(eng:set_default('ECDSA'))
+      end
     end
 
     function TestEngine:testLoop()
