@@ -60,7 +60,6 @@ TestObject = {}
         assertEquals(txt, ln)
         assertEquals(o1:txt(false), txt)
 
-        assertErrorMsgContains('(need accept paramater)', asn1.new_object)
         local options = {
             oid ='1.2.840.10045.2.1.2.1',
             sn  ='gmsm21',
@@ -97,8 +96,6 @@ TestObject = {}
             assertNil(openssl.error())
         else
             assert(asn1.txt2nid(self.ne_oid))
-
-
             assert(asn1.new_object(self.ne_sn))
             assert(asn1.new_object(self.ne_ln))
             assert(asn1.new_object(self.ne_oid))
@@ -159,3 +156,17 @@ TestTime = {}
         local t1 = at:get(self.time)
         assertEquals(self.time,t1)
     end
+
+TestNumber = {}
+    function TestNumber:testBasic()
+        local i = 0
+        local n = asn1.new_integer(i):i2d()
+        assert(n)
+        local m = asn1.new_integer()
+        m:d2i(n)
+        assert(m:i2d()==n)
+        n = asn1.new_integer():i2d()
+        assert(n)
+        n = asn1.new_integer(nil):i2d()
+        assert(n)
+    end
