@@ -20,7 +20,7 @@ function TestX509:testNew()
         assert(cacert:parse().ca, 'invalid ca certificate')
 
         local c = cacert:pubkey():encrypt('abcd')
-        d = pkey:decrypt(c)
+        local d = pkey:decrypt(c)
         assert(d=='abcd')
         assert(cacert:check(pkey),'self sign check failed')
         local castore = openssl.x509.store.new({cacert})
@@ -29,13 +29,13 @@ function TestX509:testNew()
         --sign cert by cacert
 
         local dkey = openssl.pkey.new()
-        req = assert(csr.new(self.dn,dkey))
+        local req = assert(csr.new(self.dn,dkey))
         local cert = openssl.x509.new(2,req)
         cert:notbefore(os.time())
         cert:validat(os.time(), os.time() + 3600*24*360)
         assert(cert:sign(pkey,cacert))
 
-        local c = cert:pubkey():encrypt('abcd')
+        c = cert:pubkey():encrypt('abcd')
         d = dkey:decrypt(c)
         assert(d=='abcd')
         assert(cert:check(dkey),'self private match failed')
