@@ -33,7 +33,10 @@ dofile('ec.lua')
 dofile('sm2.lua')
 
 --LuaUnit.verbosity = 0
-LuaUnit.run()
-print(openssl.error(true))
+local runner = LuaUnit.new()
+runner:setOutputType("tap")
+local retcode = runner:runSuite()
+print(openssl.print_errors():get_mem())
 collectgarbage()
+os.exit(retcode)
 
